<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:nk="http://netkernel.org">
    <xsl:output method="xml"/>
    <xsl:param name="request" nk:class="java.lang.String" />
    <xsl:param name="mode" nk:class="java.lang.String" />
    <xsl:param name="header" />
    
	<xsl:variable name="trace-view">/evisualizer</xsl:variable>
	<xsl:variable name="trace-raw">/evisualizer</xsl:variable>
	<xsl:variable name="imgbase">/nkse/style/img/</xsl:variable>
	<xsl:variable name="explorer">/tools/explore/</xsl:variable>
	
	<xsl:variable name="duration" select="/capture/trace/end"/>
	
	<xsl:variable name="showTimings" select="count(/*//trace[e-duration])>0"/>
	
	<xsl:template match="/*" xmlns:xrl="http://netkernel.org/xrl" >
		
		<xsl:variable name="foundRequest" select=".//trace[index=$request]"/>
		<div>
		
				<xsl:if test="not($mode='subrequests')">
					<xrl:include>
						<xrl:identifier>active:aeMenu</xrl:identifier>
						<xrl:argument name="breadcrumbs">
							<breadcrumbs>
								<crumb name="Visualizer" view="visualizer"/>
								<crumb name="{trace/trace/identifier}" type="trace" entity="{hash}"/>
								<xsl:if test="$request">
									<crumb type="request" entity="{hash}/{$foundRequest/index}">
										<xsl:choose>
											<xsl:when test="$foundRequest/rfe">
												<xsl:attribute name="name">[resolution failure]</xsl:attribute>
											</xsl:when>
											<xsl:when test="$foundRequest/cached">
												<xsl:attribute name="name">[from cache]</xsl:attribute>
											</xsl:when>
											<xsl:when test="$foundRequest/internalDependency">
												<xsl:attribute name="name">[internal dependency]</xsl:attribute>
											</xsl:when>
											<xsl:otherwise>
												<xsl:attribute name="name"><xsl:value-of select="$foundRequest/endpointShort"/></xsl:attribute>
											</xsl:otherwise>
										</xsl:choose>
									</crumb>
								</xsl:if>
							</breadcrumbs>
						</xrl:argument>
					</xrl:include>
				</xsl:if>
				
				<xsl:if test="$header">
					<xsl:copy-of select="$header"/>
				</xsl:if>
				
				
				<script type="text/javascript">
				<xsl:comment>
				
				$(document).ready(function()
				{	
					initDataTable("tt2" ,
					{	"aaSorting": [],
						"bLengthChange": false,
						<xsl:if test="$mode">
						"bFilter": false,
						"bInfo": false,
						"bPaginate": false,
						<!--"bSort": false,-->
						</xsl:if>
						"aoColumnDefs": [ 
							{ "bSearchable": false, "aTargets": [ 0,5,6 ] },
							{ "bVisible": false, "aTargets": [ 0,5,7 ] },
							<xsl:if test="not($showTimings)">
							{ "bVisible": false, "aTargets": [ 6,8 ] },
							</xsl:if>
							<xsl:if test="$mode='subrequests'">
							{ "bVisible": false, "aTargets": [ 0,2,5,7,8 ] },
							</xsl:if>
							<xsl:if test="$mode='determinants'">
							{ "bVisible": false, "aTargets": [ 0,2,5,6,7,8,10 ] },
							</xsl:if>
							<xsl:if test="$mode='scopedet'">
							{ "bVisible": false, "aTargets": [ 0,2,5,6,7,8,9,10 ] },
							</xsl:if>
							{ "bSortable": false, "aTargets": [ 2,3,4,9,10 ] },
							{ "iDataSort": 0, "aTargets": [ 1 ] },
							{ "iDataSort": 5, "aTargets": [ 6 ] },
							{ "iDataSort": 7, "aTargets": [ 8 ] },
							{ "asSorting": [ "asc" ], "aTargets": [ 1 ] },
							{ "asSorting": [ "desc" ], "aTargets": [ 5,6,7,8 ] },
							{ "sType": "numeric", "aTargets": [ 5,6,7,8 ] }
						]
					}  );
					initLinksRaw('.el-request','gMenuRequest');
					initLinksRaw('.el-trace','gMenuTrace');
					var width=$(window).width()-64;
					$("#tt2").css("width",width+"px");
					$("#tt2").css("display","");
					
					$(".expand").click(function(event)
					{	var ei=$(event.currentTarget).attr("ei");
						event.stopPropagation();
						handleExpand(event,"j"+ei);
					});					
					
				});
					
					function handleOver(e,id)
					{ 
var posx = 0;
	var posy = 0;
	if (!e) var e = window.event;
	if (e.pageX || e.pageY) 	{
		posx = e.pageX;
		posy = e.pageY;
	}
	else if (e.clientX || e.clientY) 	{
		posx = e.clientX + document.body.scrollLeft
			+ document.documentElement.scrollLeft;
		posy = e.clientY + document.body.scrollTop
			+ document.documentElement.scrollTop;
	}					
					
						e=document.getElementById(id);
					    e.style.top=posy+20;
						e.style.display= ""
					}

					function handleOut(id)
	                { document.getElementById(id).style.display= "none"
					}
					
					function getNamedChild(node,childName)
					{	if (node)
						{	for(child=node.firstChild; child; child=child.nextSibling )
							{	//console.log(child.nodeName);
								if (child.nodeType==1 &amp;&amp; child.nodeName==childName)
								{	return child;
								}
							}
						}
					}

					function handleExpand(event, id)
					{							
						tr=document.getElementById(id);
						tab=tr.parentNode.parentNode;
						d=tr.getAttribute("depth");
						d=parseInt(d);
						img=$(tr).find("td > div > a > img");
						//img=getNamedChild(getNamedChild(getNamedChild(getNamedChild(tr,"TD"),"DIV"),"A"),"IMG");
					    //expand=img.src.match("plus.png")
					    expand=img.attr("src").match("plus.png");
						if (expand)
						{  
							img.attr("src","<xsl:value-of select="$imgbase"/>minus.png");
							for(sib=tr.nextSibling; sib; sib=sib.nextSibling )
							{   if (sib.nodeType==1 )
								{	sibd=sib.getAttribute("depth");	
									if (sibd)
									{	//console.log(sibd);
										sibd=parseInt(sibd);
										if (sibd > d )
										{	sib.style.display="";
										}
										else if (sibd &lt;= d)
										{	break;
										}
										//fix [+] buttons - all must be expanded now
										img=getNamedChild(getNamedChild(getNamedChild(getNamedChild(sib,"TD"),"DIV"),"A"),"IMG");
										if (img)
										{	expand2=img.src.match("plus.png");
											if (expand2)
											{	img.src="<xsl:value-of select="$imgbase"/>minus.png";
											}
										}
									}
								}
							}
							tab.style.border.right="solid 1px transparent";
						}
						else
						{  //img.src="/tools/introspect/pub/plus.png";
							img.attr("src","<xsl:value-of select="$imgbase"/>plus.png");
							for(sib=tr.nextSibling; sib; sib=sib.nextSibling )
							{   if (sib.nodeType==1 )
								{	sibd=sib.getAttribute("depth");	
									if (sibd)
									{	//console.log(sibd);
										if (sibd > d)
										{	sib.style.display="none";
										}
										else if (sibd &lt;= d)
										{	break;
										}
									}
								}
							}
							tab.style.border.right="none";
						}
					
						//reflow striping
						/*
						even=tr.className.match("evenrow");
						for(sib=tr.nextSibling; sib; sib=sib.nextSibling )
						{   if (sib.nodeType==1 &amp;&amp; sib.style.display=="")
							{  
								if (even)
								{	sib.className="oddrow";
									even=false;
								}
								else
								{	sib.className="evenrow";
									even=true;
								}
							}
						}
						*/
						return true;
					}
				</xsl:comment>
				</script>
				
				<table style="width:100%; clear: both; font-size: 9px; display: none;" id="tt2" >
					<thead>	<tr>
						<th>Index</th>
						<th>Physical Endpoint / Callstack</th>
						<th>Space</th>
						<th>Verb</th>
						<th>Request Identifier</th>
						<th>DurationRaw</th>
						<th>Duration (ms)</th>
						<th>CPURaw</th>
						<th>CPU (ms)</th>
						<th>Cache</th>
						<th>Response Representation</th>
						<xsl:if test="$mode='scopedet'">
							<th>Scope</th>
						</xsl:if>
					</tr></thead>
					
					<xsl:choose>
						<xsl:when test="$mode='callstack'">
							<xsl:for-each select="trace">
								<xsl:call-template name="trace">
									<xsl:with-param name="duration" select="trace/e-duration"/>
									<xsl:with-param name="depth">0</xsl:with-param>
									<xsl:with-param name="dmap"></xsl:with-param>
								</xsl:call-template>
							</xsl:for-each>
						</xsl:when>
						<xsl:when test="$request">
							<xsl:for-each select=".//trace[index=$request]">
								<xsl:call-template name="trace">
									<xsl:with-param name="duration" select="e-duration"/>
									<xsl:with-param name="depth">0</xsl:with-param>
									<xsl:with-param name="dmap"></xsl:with-param>
								</xsl:call-template>
							</xsl:for-each>
						</xsl:when>
						<xsl:otherwise>
							<xsl:for-each select="trace">
								<xsl:call-template name="trace">
									<xsl:with-param name="duration" select="trace/e-duration"/>
									<xsl:with-param name="depth">0</xsl:with-param>
									<xsl:with-param name="dmap"></xsl:with-param>
								</xsl:call-template>
							</xsl:for-each>
						</xsl:otherwise>
					</xsl:choose>
				</table>
		</div>
	</xsl:template>
	
	<xsl:template name="tree">
		<xsl:param name="dmap" />
		<xsl:param name="children" />
		<xsl:if test="$dmap != ''">
			<xsl:variable name="letter" select="substring($dmap, 1, 1)" />
			<xsl:call-template name="tree-el">
				<xsl:with-param name="el"><xsl:value-of select="$letter" /></xsl:with-param>

				<xsl:with-param name="last">
					<xsl:choose>
						<xsl:when test="string-length($dmap)=1 and string-length($children)"><xsl:value-of select="$children" /></xsl:when>
						<xsl:when test="string-length($dmap)=1">0</xsl:when>
						<xsl:otherwise></xsl:otherwise>
					</xsl:choose>
				</xsl:with-param>
			
			</xsl:call-template>
			<xsl:call-template name="tree">
				<xsl:with-param name="dmap" select="substring-after($dmap, $letter)" />
				<xsl:with-param name="children" select="$children" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="tree-el">
		<xsl:param name="el"/>
		<xsl:param name="last"/>
		<xsl:choose>
			<xsl:when test="string-length($last)=0">
				<xsl:choose>
					<xsl:when test="$el=0"><span class="uit1" ><xsl:text> </xsl:text></span></xsl:when>
					<xsl:otherwise><span class="uit0"><xsl:text> </xsl:text></span></xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<span>
					<xsl:choose>
						<xsl:when test="$el=0"><xsl:attribute name="class">uit2</xsl:attribute></xsl:when>
						<xsl:otherwise><xsl:attribute name="class">uit3</xsl:attribute></xsl:otherwise>
					</xsl:choose>
					<xsl:text> </xsl:text>
				</span>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	
	
	<xsl:template match="trace" name="trace">
		<xsl:param name="depth"/>
		<xsl:param name="dmap"/>
		<xsl:param name="duration"/>

		<tr depth="{$depth}" id="j{index}">
			<xsl:if test="index>=0">
				<xsl:attribute name="class">el el-request</xsl:attribute>
				<xsl:attribute name="entity"><xsl:value-of select="/*/hash"/>/<xsl:value-of select="index"/></xsl:attribute>
			</xsl:if>
			<xsl:if test="$mode='determinants' and determinant">
				<xsl:attribute name="class">el el-request expiry-determinant</xsl:attribute>
			</xsl:if>
			<xsl:if test="$mode='scopedet' and determinant">
				<xsl:attribute name="class">el el-request scope-determinant</xsl:attribute>
			</xsl:if>
		
			<td><xsl:value-of select="index"/></td>
			
			<td >
				<div class="uitw"><span class="uit">
					<xsl:call-template name="tree">
						<xsl:with-param name="dmap"><xsl:value-of select="$dmap"/></xsl:with-param>
						<xsl:with-param name="children">
							<xsl:if test="count(trace)"><xsl:value-of select="index"/></xsl:if>
						</xsl:with-param>
					</xsl:call-template>
					</span>
									
					<xsl:if test="trace and not($mode)">
						<a class="expand" ei="{index}"><img src="{$imgbase}minus.png" border="0"/></a>
					</xsl:if>
					
					<xsl:choose>
						<xsl:when test="rfe">
							<span class="ui-icon-ae ui-icon-ae-rfe"><xsl:text> </xsl:text></span>
							[none]
						</xsl:when>
						<xsl:when test="cached">
							<span class="ui-icon-ae ui-icon-ae-cached"><xsl:text> </xsl:text></span>
							[from cache]
						</xsl:when>
						<xsl:when test="internalDependency">
							<span class="ui-icon-ae ui-icon-ae-cached"><xsl:text> </xsl:text></span>
							[internal dependency]
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="endpointShort"/>
						</xsl:otherwise>
					</xsl:choose>
					
					<!--
					(<xsl:value-of select="index"/>)
					<xsl:if test="$mode='scopedet' and determinant">
					</xsl:if>
					(<xsl:value-of select="reqctxDepth - scope - 1"/>,<xsl:value-of select="reqctxDepth"/>)
					-->
				</div>
			</td>
			
			<td>
				<xsl:choose>
					<xsl:when test="endpointHomeHash">
						<xsl:value-of select="endpointHomeContext"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="endpointHomeContext"/>
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td><xsl:value-of select="verb"/>
			<xsl:if test="async='true'"> (async)</xsl:if>
			</td>
			<td>
				<xsl:call-template name="wrapped">
					<xsl:with-param name="text"><xsl:value-of select="identifier"/></xsl:with-param>
				</xsl:call-template>
			</td>
			<xsl:choose>
			<xsl:when test="$showTimings">
				<td>
					<xsl:if test="e-duration"><xsl:value-of select="e-duration"/></xsl:if>
				</td>
				<td>
					<xsl:if test="e-duration">
						<xsl:variable name="perc" select="e-duration div ($duration)"/>
						<div class="dur" style="width: {100*$perc}%">
							<!--<div><xsl:value-of select='format-number(e-duration, "0.#")' /></div>-->
							<div><xsl:value-of select='format-number(e-duration, "0")' /></div>
						</div>
					</xsl:if>
				</td>
				<td>
					<xsl:if test="t-duration"><xsl:value-of select="t-duration"/></xsl:if>
				</td>
				<td>
					<xsl:if test="t-duration">
						<xsl:variable name="perc" select="t-duration div ($duration)"/>
						<div class="dur" style="width: {100*$perc}%">
							<div><xsl:value-of select='format-number(t-duration, "0.00")' /></div>
						</div>
					</xsl:if>
				</td>
			</xsl:when>
			<xsl:otherwise>
				<td/>
				<td/>
				<td/>
				<td/>
			</xsl:otherwise>
			</xsl:choose>
			<td>
				<xsl:choose>
					<xsl:when test="isCachable='true'">
						<div class="valid">&#160;</div>
					</xsl:when>
					<xsl:when test="isCachable='false'">
						<div class="expired">&#160;</div>
					</xsl:when>
					<xsl:when test="isCachable='maybe'">
						<div style="background: #D0B000;">&#160;</div>
					</xsl:when>
				</xsl:choose>
			</td>
			<td>
				<xsl:if test="responseException='true'">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
				<xsl:choose>
					<xsl:when test="responseClassShort='null'">
						<xsl:value-of select="responseClassShort"/>
					</xsl:when>
					<xsl:when test="responseName">
						<xsl:value-of select="responseName"/> 
						<!--
						<a href="{$trace-raw}/{/capture/hash}/response/{$count}">
							<xsl:value-of select="responseName"/> 
						</a>
						-->
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="responseClassShort"/>
						<!--
						<a href="{$trace-raw}/{/capture/hash}/response/{$count}">
							<xsl:value-of select="responseClassShort"/>
						</a>
						-->
					</xsl:otherwise>
				</xsl:choose>
			</td>
			
			<xsl:if test="$mode='scopedet'">
				<td>
					<xsl:if test="determinant">
					<!--
					<table style="border: solid 1px black; border-spacing: 0px;"><tr>
						<td style="padding: 0px;"><img style="background: #000;" src="/nkse/style/img/blank.png" width="{determinant*50}" height="10" border="0"/></td>
						<td style="padding: 0px;"><img src="/nkse/style/img/blank.png" width="{(1-determinant)*50}" height="10" border="0"/></td>						
					</tr></table>
					-->
						<xsl:variable name="in" select="substring-before(determinant,':')"/>
						<xsl:variable name="out" select="substring-after(determinant,':')"/>
						<!--<xsl:value-of select="$in"/>@<xsl:value-of select="$out"/>-->
						<xsl:choose>
							<xsl:when test="$out &gt; 0">
								<span class="in">
									<xsl:call-template name="many">
										<xsl:with-param name="count" select="$in"/>
										<xsl:with-param name="template">
											<span class="ui-icon-ae ui-icon-ae-hscope"><xsl:text> </xsl:text></span>
										</xsl:with-param>
									</xsl:call-template>
								</span>
								<span class="out">
									<xsl:call-template name="many">
										<xsl:with-param name="count" select="$out - 1"/>
										<xsl:with-param name="template">
											<span class="ui-icon-ae ui-icon-ae-hscope"><xsl:text> </xsl:text></span>
										</xsl:with-param>
									</xsl:call-template>
									<span class="ui-icon-ae ui-icon-ae-hscope-end"><xsl:text> </xsl:text></span>
								</span>
							</xsl:when>
							<xsl:otherwise>
								<span class="in">
									<xsl:call-template name="many">
										<xsl:with-param name="count" select="$in - 1"/>
										<xsl:with-param name="template">
											<span class="ui-icon-ae ui-icon-ae-hscope"><xsl:text> </xsl:text></span>
										</xsl:with-param>
									</xsl:call-template>
									<span class="ui-icon-ae ui-icon-ae-hscope-end"><xsl:text> </xsl:text></span>
								</span>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:if>
				</td>
			</xsl:if>
		</tr>
		
		<xsl:choose>
			<xsl:when test="$mode='callstack'">
				<xsl:for-each select="trace[.//trace[index=$request] or index=$request]">
					<xsl:call-template name="trace">
						<xsl:with-param name="depth"><xsl:value-of select="$depth + 1"/></xsl:with-param>
						<xsl:with-param name="dmap"><xsl:value-of select="$dmap"/>1</xsl:with-param>
						<xsl:with-param name="duration" select="$duration"/>
					</xsl:call-template>
				</xsl:for-each>

			</xsl:when>
			<xsl:when test="$mode='subrequests'">
				<xsl:if test="$depth &lt; 1">
					<xsl:for-each select="trace">
						<xsl:variable name="c">
							<xsl:choose>
								<xsl:when test="position()=last()">1</xsl:when>
								<xsl:otherwise>0</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
							
						<xsl:call-template name="trace">
							<xsl:with-param name="depth"><xsl:value-of select="$depth + 1"/></xsl:with-param>
							<xsl:with-param name="dmap"><xsl:value-of select="$dmap"/><xsl:value-of select="$c"/></xsl:with-param>
							<xsl:with-param name="duration" select="$duration"/>
						</xsl:call-template>
					</xsl:for-each>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="trace[not(internalDependency)]">
					<xsl:variable name="c">
						<xsl:choose>
							<xsl:when test="position()=last()">1</xsl:when>
							<xsl:otherwise>0</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
						
					<xsl:call-template name="trace">
						<xsl:with-param name="depth"><xsl:value-of select="$depth + 1"/></xsl:with-param>
						<xsl:with-param name="dmap"><xsl:value-of select="$dmap"/><xsl:value-of select="$c"/></xsl:with-param>
						<xsl:with-param name="duration" select="$duration"/>
					</xsl:call-template>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
			
	</xsl:template>	
	
	<xsl:template name="many">
		<xsl:param name="count"/>
		<xsl:param name="template"/>
		<xsl:if test="$count &gt; 0">
			<xsl:copy-of select="$template"/>
			<xsl:call-template name="many">
				<xsl:with-param name="count" select="$count - 1"/>
				<xsl:with-param name="template" select="$template"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template name="wrapped">
		<xsl:param name="text"/>
		<xsl:choose>
			<xsl:when test="string-length($text)>20">
				<xsl:value-of select="substring($text,1,20)"/><span class="breakspace"><xsl:text> </xsl:text></span><xsl:call-template name="wrapped">
					<xsl:with-param name="text" select="substring($text,20+1)"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$text"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
</xsl:stylesheet>
