<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml"/>
	<xsl:variable name="traces-view">/visualizer/</xsl:variable>
	<xsl:variable name="imgbase">/nkse/style/img/</xsl:variable>
	<xsl:param name="param"><sortBy>timestamp</sortBy></xsl:param>
	
	<xsl:template match="/trace" xmlns:xrl="http://netkernel.org/xrl" >
	
		
		
		<div>
		
			<xrl:include>
				<xrl:identifier>active:aeMenu</xrl:identifier>
				<xrl:argument name="breadcrumbs">
					<breadcrumbs>
						<crumb name="Visualizer" view="visualizer"/>
					</breadcrumbs>
				</xrl:argument>
			</xrl:include>

			<xrl:include>
			 <xrl:identifier>active:controlPanelInfoBox</xrl:identifier>
			 <xrl:argument name="content">
			   <div>
           <p>
             The Visualizer captures complete trace information about requests processed by NetKernel.
           </p>
           <p>
             The Visualizer captures a large quantity of information and is normally <i>off</i>
             to converse memory.
             To turn the Visualizer <i>on</i>, click the button labeled &quot;start&quot;.
             To turn the Visualizer <i>off</i>, click the button labeled &quot;stop&quot;.
             A regular expression filter may be entered to limit the requests captured
             while the Visualizer is on.
             The button labeled &quot;refresh&quot; will update the display to show all
             captured requests.
           </p>
           <p>
             To explore the details of a particular captured request,
             click on the root request identifier.
             This will display full details including each sub-request,
             returned representations and more.
					</p>
					<p>
            Refer to the
            <a href="/book/view/book:guide:logicalreference/doc:logicalreference:tools:visualizer">Visualizer Guide</a>
            for more details.
					</p>
			   </div>
			 </xrl:argument>
			</xrl:include> 
		
			<script language="Javascript"><xsl:comment>
				function doSubmit(action2)
				{	document.form.action2.value=action2;
					
					v=document.form.match.value;
					if (v=='Root request regex filter...')
					{	document.form.match.value='';
					}
					document.form.submit();
				}
				
				$(document).ready(function()
				{	
					initDataTable("tt2",
					{ 'bSort': true, 'aaSorting': [[2,'asc']], 'iDisplayLength': 25,
						"aoColumnDefs": [ 
							{ "bSortable": false, "aTargets": [ 4 ] },
							{ "bVisible": false, "aTargets": [ 0 ] },
							{ "iDataSort": 0, "aTargets": [ 2 ] }
						],
						 "fnDrawCallback": bind
					} );
				
					$(".delete").click(function(event)
					{	var ei=$(event.currentTarget).attr("ei");
						event.stopPropagation();
						doMenu("visTraceDelete",ei);
					});
				
				});
				function bind()
				{	initLinksRaw('#tt2 .el-trace','gMenuTrace');
				}
				
				
			</xsl:comment></script>
				<form method="post" enctype="multipart/form-data" name="form">
					<input type="hidden" name="action2" value=""/>
					<table class="form-table">
						<tr>
							<td>
								<xsl:choose>
									<xsl:when test="enabled='true'">
										<input type="button" onclick="doSubmit('stop')" value="stop" class="btn"/>
									</xsl:when>
									<xsl:otherwise>
										<input type="button" onclick="doSubmit('start')" value="start" class="btn"/>
									</xsl:otherwise>
								</xsl:choose>
							</td><td>
								<input type="button" onclick="doSubmit('clear')" value="delete all" class="btn"/>
							</td>
							<td>
								<input type="text" name="match" value="{match}">
									<xsl:choose>
										<xsl:when test="string-length(match)">
											<xsl:attribute name="value"><xsl:value-of select="match"/></xsl:attribute>
										</xsl:when>
										<xsl:otherwise>
											<xsl:attribute name="value">Root request regex filter...</xsl:attribute>
											<xsl:attribute name="onclick">value=''</xsl:attribute>
											<xsl:attribute name="onfocus">value=''</xsl:attribute>
											<xsl:attribute name="class">input-tip</xsl:attribute>
										</xsl:otherwise>
									</xsl:choose>
								</input>
							</td>
							<td>
								<input type="button" onclick="doSubmit('update')" value="refresh" class="btn"/>
							</td>
							<td>
								<input type="button" onclick="doSubmit('cache')" value="clear cache" class="btn"/>
							</td>
							<xsl:if test="impl!='org.netkernel.layer0.tools.EndpointMonitor'">
								<td>
									Import Saved Trace State:
								</td><td>
									<input type="file" name="uploaded"/>
								</td>
								<td>
									<input type="button" onclick="doSubmit('upload')" value="upload"  class="btn"/>
								</td>
							</xsl:if>
						</tr>
					</table>
				</form>
				
				
				<table style="width:100%;" id="tt2">
					<thead>	<tr>
						<th>TS Raw</th>
						<th>Root Request Identifier</th>
						<th>Timestamp</th>
						<th>Duration</th>
						<th></th>
					</tr></thead>
					<xsl:for-each select="captured/trace">
						<xsl:call-template name="trace2"/>
					</xsl:for-each>
				</table>
				<br/>

		</div>		
	</xsl:template>
	
	<xsl:template name="trace2">
		<tr class="el el-trace" entity="{hash}">
			<td><xsl:value-of select="timestamp_raw"/></td>
			<td>
				<span class="ui-icon-ae ui-icon-ae-trace"><xsl:text> </xsl:text></span>
				<xsl:value-of select="request"/>
				<xsl:if test="imported"> [imported]</xsl:if>
			</td>
			<td><xsl:value-of select="timestamp"/></td>
			<td align="right"><xsl:value-of select="duration"/></td>
			<td>
				<img class="delete" ei="{hash}" src="{$imgbase}delete.png" border="0"/>
				<!--<img border="0" src="{$imgbase}delete.png" onclick="doMenu('visTraceDelete','{hash}')"/>-->
			</td>
		</tr>
	</xsl:template>


	
	
</xsl:stylesheet>
