package org.ten60.netkernel.visualizer;

import java.io.File;
import java.io.FileOutputStream;
import java.util.zip.*;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.xml.xda.IXDAReadOnly;

public class LoadedTrace
{
	private final ZipFile mZip;
	private final long mDuration;
	private long mTimestamp;
	private String mHash;
	private final String mRootRequest;
	
	public LoadedTrace(IBinaryStreamRepresentation aData, INKFRequestContext aHelper) throws Exception
	{
		File tmp = File.createTempFile("visualizer", ".zip");
		FileOutputStream os = new FileOutputStream(tmp);
		aData.write(os);
		os.close();
		mZip = new ZipFile(tmp);
		
		IXDAReadOnly traceDoc=(IXDAReadOnly)aHelper.transrept(getEntry("trace.xml"), IXDAReadOnly.class);
		mDuration=Long.parseLong(traceDoc.getText("duration", true));
		try
		{	mTimestamp=Long.parseLong(traceDoc.getText("timestamp_raw", true));
		} catch (Exception e)
		{	mTimestamp=0;
		}
		mHash=traceDoc.getText("hash", true);
		mRootRequest=traceDoc.getText("request", true);
	}
	
	public void newHash()
	{	int hash = Integer.parseInt(mHash, 16);
		hash++;
		mHash = Integer.toHexString(hash);
	}
	
	private IBinaryStreamRepresentation getEntry(String aName)
	{	IBinaryStreamRepresentation result=null;
		ZipEntry entry=mZip.getEntry(aName);
		if (entry!=null)
		{	result = new ZipFileEntryRepresentation(mZip,entry);
		}
		return result;
	}
	
	public long getDuration()
	{	return mDuration;
	}
	
	public long getTimestamp()
	{	return mTimestamp;
	}
	
	public String getHash()
	{	return mHash;
	}
	
	public String getRootRequest()
	{	return mRootRequest;
	}
	
	public IBinaryStreamRepresentation getTraceView(INKFRequestContext aHelper) throws NKFException
	{	return getEntry("index.xml");
	}
	
	public IBinaryStreamRepresentation getTraceDetail(INKFRequestContext aHelper, String aRequestId) throws NKFException
	{
		String resource = "detail/"+aRequestId+".xml";
		return getEntry(resource);
	}
	
	public INKFResponse getResponse(INKFRequestContext aHelper, String aRequestId, boolean aPrimary) throws Exception
	{	String dir=aPrimary?"primary/":"response/";
		String resource=dir+aRequestId;
		IBinaryStreamRepresentation rep= getEntry(resource);
		INKFResponse response=aHelper.createResponseFrom(rep);
		resource = dir+aRequestId+".mime";
		String mime=(String)aHelper.transrept(getEntry(resource), String.class);
		response.setMimeType(mime);		
		return response;
	}
}
