package org.ten60.netkernel.visualizer;

import org.netkernel.container.IKernel;
import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.layer0.tools.EndpointMonitor;
import org.netkernel.layer0.tools.EndpointMonitor.RequestCapture;
import org.netkernel.layer0.tools.EndpointMonitor.RequestTrace;
import org.netkernel.layer0.util.*;
import org.netkernel.module.standard.StandardSpace;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IResponse;
import org.netkernel.request.IResponseMeta;
import org.netkernel.request.impl.ExpiryFactory;
import org.netkernel.request.impl.RequestFactory;
import org.netkernel.scheduler.IInternalResponseMeta;
import org.netkernel.scheduler.ResolutionFailureException;
import org.netkernel.scheduler.TraceDebugCapture;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IMetaRepresentation;
import org.netkernel.urii.ISpace;
import org.netkernel.util.SynchronousRequestee;
import org.w3c.dom.Document;

import java.io.*;
import java.text.*;
import java.util.*;
import java.util.zip.*;

public class VisualizerAccessor extends StandardAccessorImpl
{
	private SimpleDateFormat mTimestampFormat = new java.text.SimpleDateFormat("d-MMM-yyyy HH:mm:ss.SSS");
	private static final String VISUALIZER_IGNORE="visualizer_ignore";
	private List<LoadedTrace> mLoadedTraces;
	private static NumberFormat sDecimalFormat = new DecimalFormat("0.##");
	private static String NULL_STRING="null";
	private boolean mTimings;
	
	/**
	 * 
	 */
	public VisualizerAccessor()
	{	declareThreadSafe();
		declareDescription("visualizes stuff");
	}
	
	/* (non-Javadoc)
	 * @see org.netkernel.layer0.nkf.impl.NKFEndpointImpl#onSource(org.netkernel.layer0.nkf.INKFRequestContext)
	 */
	public void onSource(INKFRequestContext aHelper) throws Exception
	{
		EndpointMonitor monitor=(EndpointMonitor)Layer0Factory.getMonitorInstance();
		if (monitor==null)
		{	throw new NKFException("EndpointMonitor not initialised");
		}
		mTimings=monitor.getClass()!=EndpointMonitor.class;
		
		IRepDeployedModules dmr=(IRepDeployedModules)aHelper.source("active:moduleList", IRepDeployedModules.class);
		
		String action=aHelper.getThisRequest().getArgumentValue("action");
		if (action==null)
		{	action=aHelper.getThisRequest().getArgumentValue("action2");
		}
		if (action.equals("Data"))
		{	INKFRequestReadOnly req = aHelper.getThisRequest();
			String trace=req.getArgumentValue("trace");
			if (trace==null)
			{	generateTracesView(aHelper,monitor);
			}
			else
			{
				if (req.argumentExists("detail"))
				{	String requestId = req.getArgumentValue("detail");
					generateDetailView(aHelper,monitor,trace,requestId,dmr);
				}
				else if (req.argumentExists("response"))
				{	String requestId = req.getArgumentValue("response");
					generateResponseView(aHelper,monitor,trace,requestId,false);
				}
				else if (req.argumentExists("primary"))
				{	String requestId = req.getArgumentValue("primary");
					generateResponseView(aHelper,monitor,trace,requestId,true);
				}
				else if (req.argumentExists("save"))
				{	generateSaveResponse(aHelper,monitor,trace,dmr);
				}
				else if (req.argumentExists("full"))
				{	System.out.println("FULL");
					generateTraceView(aHelper,monitor,trace,dmr,true);
				}
				else
				{	generateTraceView(aHelper,monitor,trace,dmr,false);
				}
			}
		}
		else if (action.equals("Start"))
		{	monitor.enableTrace(true);
		}
		else if (action.equals("Stop"))
		{	monitor.enableTrace(false);
		}
		else if (action.equals("Clear"))
		{	monitor.clearCapturedTraces();
			mLoadedTraces=null;
		}
		else if (action.equals("SetFilter"))
		{	String filter = (String)aHelper.source("arg:data",String.class);
			monitor.setRootTraceMatch(filter);
		}
		else if (action.equals("Load"))
		{	IBinaryStreamRepresentation rep=(IBinaryStreamRepresentation)aHelper.source("arg:data",IBinaryStreamRepresentation.class);
			LoadedTrace lt = new LoadedTrace(rep,aHelper);
			if (innerGetTraceForHash(monitor, lt.getHash())!=null || getLoadedTrace(lt.getHash())!=null)
			{	throw aHelper.createFormattedException("EX_TRACE_DUPLICATE", "MSG_TRACE_DUPLICATE", null, null);
			}
			if (mLoadedTraces==null)
			{	mLoadedTraces = new ArrayList<LoadedTrace>();
			}
			mLoadedTraces.add(lt);
		}
		else if (action.equals("Delete"))
		{	String traceId = aHelper.getThisRequest().getArgumentValue("data");
			deleteTrace(monitor,traceId);
		}
	}
	
	private void generateTracesView(INKFRequestContext aHelper, EndpointMonitor aMonitor) throws Exception
	{
		StringWriter sw= new StringWriter(2048);
		sw.write("<?xml version='1.0' encoding='UTF-8'?>");
		sw.write("<trace>");
		
		XMLUtils.writeEscaped(sw,"enabled",Boolean.toString(aMonitor.isTraceEnabled()));
		XMLUtils.writeEscaped(sw,"match",aMonitor.getRootTraceMatch());
		XMLUtils.writeEscaped(sw,"impl",aMonitor.getClass().getName());
		
		sw.write("<captured>");
		
		RequestTrace[] traces=aMonitor.getCapturedTraces();
		for (int i=0; i<traces.length; i++)
		{	
			RequestTrace t=traces[i];
			RequestCapture r=t.getRoot();
			if (r.getResponse()!=null && !Boolean.TRUE.equals(r.getRequestIn().getHeaders().getValue(VISUALIZER_IGNORE)))
			{	writeTraceSummary(sw,t);
			}
		}
		
		if (mLoadedTraces!=null)
		{
			for (int i=0; i<mLoadedTraces.size(); i++)
			{	
				LoadedTrace lt = (LoadedTrace)mLoadedTraces.get(i);
				
				sw.write("<trace>");
				XMLUtils.writeEscaped(sw,"timestamp_raw",Long.toString(lt.getTimestamp()) );
				XMLUtils.writeEscaped(sw,"timestamp",mTimestampFormat.format(new Date(lt.getTimestamp())) );
				XMLUtils.writeEscaped(sw,"duration",Long.toString(lt.getDuration() ));
				XMLUtils.writeEscaped(sw,"request", lt.getRootRequest()  );
				XMLUtils.writeEscaped(sw,"hash", lt.getHash()  );
				sw.write("<imported/>");
				sw.write("</trace>");			
			}
		}
		
		
		sw.write("</captured>");
		
		sw.write("</trace>");

		INKFResponse resp=aHelper.createResponseFrom(sw.toString());
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
		
		
	}
	
	private void writeTraceSummary(Writer aWriter, RequestTrace aTrace) throws Exception
	{
		aWriter.write("<trace>");
		long time=aTrace.getStart();
		XMLUtils.writeEscaped(aWriter,"timestamp_raw",Long.toString(time));
		XMLUtils.writeEscaped(aWriter,"timestamp",mTimestampFormat.format(new Date(time)) );
		XMLUtils.writeEscaped(aWriter,"duration",Long.toString(aTrace.getEnd()-time ));
		IRequest req=aTrace.getRoot().getRequest();
		if (req==null)
		{	req=aTrace.getRoot().getRequestIn();
		}
		XMLUtils.writeEscaped(aWriter,"request", req.getIdentifier().toString()  );
		int hash=(int)time ^ req.hashCode();
		XMLUtils.writeEscaped(aWriter,"hash", Integer.toHexString(hash)  );
		aWriter.write("</trace>");			
	}
	
	private void generateTraceView(INKFRequestContext aHelper, EndpointMonitor aMonitor, String aTraceId, IRepDeployedModules aDMR, boolean aFull) throws Exception
	{
		IBinaryStreamRepresentation traceString;
		
		RequestTrace trace = innerGetTraceForHash(aMonitor,aTraceId);
		if (trace!=null)
		{	if (trace.getRoot().getResponse()==null)
			{	throw new NKFException("trace not found");
			}
			traceString = generateTraceView(trace, aTraceId,aDMR,aFull);
		}
		else
		{	LoadedTrace lt = getLoadedTrace(aTraceId);
			if (lt!=null)
			{	traceString=lt.getTraceView(aHelper);
			}
			else
			{	throw getTraceNotFoundException(aHelper);
			}
		}
		INKFResponse resp=aHelper.createResponseFrom(traceString);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}
	
	private IReadableBinaryStreamRepresentation generateTraceView(RequestTrace aTrace, String aTraceHash, IRepDeployedModules aDMR, boolean aFull) throws Exception
	{	
		ByteArrayOutputStream baos = new ByteArrayOutputStream(2048);
		OutputStreamWriter sw = new OutputStreamWriter(baos,"UTF-8");
		
		sw.write("<?xml version='1.0'  encoding='UTF-8'?>");
		sw.write("<capture>");
		XMLUtils.writeEscaped(sw,"hash", aTraceHash );
		
		sw.write("<trace>");
		XMLUtils.writeEscaped(sw,"endpointShort",  endpointShort(aTrace.getTransport() ));
			
		sw.write("<resctx>");
		XMLUtils.writeEscaped(sw,"context", aTrace.getTransportContext().toString()  );
		sw.write("</resctx>");		
		
		writeRequest(sw,0,aTrace.getRoot(),aTrace,2,aFull,aDMR);
		sw.write("</trace>");
		sw.write("</capture>");
		
		sw.flush();
		return new ByteArrayRepresentation(baos);
	}
	
	private void generateDetailView(INKFRequestContext aHelper, EndpointMonitor aMonitor, String aTraceId, String aRequestId, IRepDeployedModules aDMR) throws Exception
	{	
		IBinaryStreamRepresentation detailString;
		RequestTrace trace = innerGetTraceForHash(aMonitor,aTraceId);
		if (trace!=null)
		{	detailString	= generateDetailViewString(aHelper,trace,aTraceId,aRequestId, aDMR);
		}
		else
		{	LoadedTrace lt = getLoadedTrace(aTraceId);
			if (lt!=null)
			{	detailString = lt.getTraceDetail(aHelper, aRequestId);
			}
			else
			{	throw getTraceNotFoundException(aHelper);
			}
		}

		INKFResponse resp=aHelper.createResponseFrom(detailString);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);			
	}
	
	private IReadableBinaryStreamRepresentation generateDetailViewString(INKFRequestContext aHelper, RequestTrace aTrace, String aTraceId, String aRequestId, IRepDeployedModules aDMR) throws Exception
	{	IKernel k=aHelper.getKernelContext().getKernel();
		ByteArrayOutputStream baos = new ByteArrayOutputStream(2048);
		OutputStreamWriter sw = new OutputStreamWriter(baos,"UTF-8");
		int index = Integer.parseInt(aRequestId);
		RequestCapture capture = getRequestCapture(aTrace,index);
		IRequest req=capture.getRequestIn();
	
		if (capture.getRequest()!=null)
		{
			if (req.getVerb()!=IRequest.VERB_TRANSREPT && capture.getRequest().getVerb()==IRequest.VERB_TRANSREPT)
			{	
				// in this secondary transrept case we don't have the actual request to resolve so we need to recreate it
				IRequestScopeLevel rctx=req.getRequestScope();
				req=RequestFactory.cloneWithScope(capture.getRequest(),rctx);
			}
		}
		else
		{	// ResolutionFailureException
			Throwable t=(Throwable)capture.getResponse().getRepresentation();
			ResolutionFailureException rfe=(ResolutionFailureException)t.getCause();
			req=rfe.getRequest();
		}
		
		sw.write("<?xml version='1.0'  encoding='UTF-8'?>");
		sw.write("<detail>");
		XMLUtils.writeEscaped(sw,"hash", aTraceId );
		XMLUtils.writeEscaped(sw,"root", aTrace.getRootRequest().getIdentifier().toString() );
		
		XMLUtils.writeEscaped(sw,"requestIndex", aRequestId );
		
		writeRequest(sw,index,capture,aTrace,1,true,aDMR);
		
		RequestCapture parentCapture=capture.getParent();
		if (parentCapture.getEndpoint()!=null)
		{	String parentEndpoint = endpointShort(parentCapture.getEndpoint());
			XMLUtils.writeEscaped(sw,"parentEndpoint",  parentEndpoint );
			XMLUtils.writeEscaped(sw,"parentRequestId",  parentCapture.getRequestIn().getIdentifier().toString() );
			XMLUtils.writeEscaped(sw,"parentRequestVerb",  RequestFactory.verbString(parentCapture.getRequestIn().getVerb()) );
			int[] indices=getCaptureIndices(aTrace, new Object[]{parentCapture});
			if (indices.length>0)
			{	XMLUtils.write(sw,"parentRequestIndex",Integer.toString(indices[0]));
			}
		}
		else
		{	XMLUtils.writeEscaped(sw,"parentEndpoint",  endpointShort(aTrace.getTransport() ));
		}
		
		TraceDebugCapture tdc = new TraceDebugCapture();
		IRequest resReq=RequestFactory.createResolveRequest(req,null,tdc,aHelper.getKernelContext().getThisKernelRequest(),null);
		try
		{	k.synchronousRequest(resReq);
		} catch (Exception e)
		{	//tdc.add("Resolution not available. Have modules been recommissioned since trace captured?");
			e.printStackTrace();
		}
		
		sw.write("<resolve>");
		Document  tdcDoc=aHelper.transrept(tdc, Document.class);
		//System.out.println(tdcHDS);
		String tdcString=XMLUtils.toXML(tdcDoc, false, true);
		sw.write(tdcString);
		//org.netkernel.util.Utils.writeTraceDebug(sw, tdc, 0);
		sw.write("</resolve>");

		sw.write("</detail>");
		sw.flush();
		return new ByteArrayRepresentation(baos);
	}
	
	private void generateResponseView(INKFRequestContext aHelper, EndpointMonitor aMonitor, String aTraceId, String aRequestId, boolean aPrimary) throws Exception
	{ 
		RequestTrace trace = innerGetTraceForHash(aMonitor,aTraceId);
		if (trace!=null)
		{
			int index = Integer.parseInt(aRequestId);
			RequestCapture req2 = getRequestCapture(trace,index);
			Object[] ram = getResponseAndMime(aHelper,req2,aPrimary);
			if (ram!=null && ram[0]!=null)
			{	aHelper.createResponseFrom(ram[0]).setMimeType((String)ram[1]);
			}
		}
		else
		{	LoadedTrace lt = getLoadedTrace(aTraceId);
			if (lt!=null)
			{	aHelper.createResponseFrom(lt.getResponse(aHelper,aRequestId,aPrimary));
			}
			else
			{	throw getTraceNotFoundException(aHelper);
			}
		}
	}
	
	private static NKFException getTraceNotFoundException(INKFRequestContext aHelper)
	{	return aHelper.createFormattedException("EX_TRACE_NOT_FOUND", "MSG_TRACE_NOT_FOUND",null,null);
	}
	
	private Object[] getResponseAndMime(INKFRequestContext aHelper, RequestCapture aRC, boolean aPrimary)
	{	
		IResponse resp;		
		if (aPrimary)
		{	Object[] values=(aRC.getRequest()!=null)?aRC.getRequest().getValues():null;
			if (values!=null && values.length>IRequest.TYPE_SOURCED_RESPONSE && values[IRequest.TYPE_SOURCED_RESPONSE]!=null)
			{	resp=(IResponse)values[IRequest.TYPE_SOURCED_RESPONSE];
			}
			else
			{	return null; // no primary
			}
		}
		else
		{	resp=aRC.getResponse();
		}
		String mime=(String)resp.getMeta().getUserMetaData().getValue(IResponseMeta.KEY_MIMETYPE);
		
		Object rep;
		if (resp.getRepresentation()!=null && !(resp.getRepresentation() instanceof IBinaryStreamRepresentation))
		{	
			try
			{	INKFRequest req=aHelper.createRequest("var:dummy");
				req.setVerb(INKFRequestReadOnly.VERB_TRANSREPRESENT);
				req.addPrimaryArgumentFromResponse(aHelper.getKernelContext().wrapKernelResponse(resp));
				req.setRepresentationClass(IBinaryStreamRepresentation.class);
				INKFResponseReadOnly resp2=aHelper.issueRequestForResponse(req);
				mime=(String)resp2.getMimeType();
				rep=resp2.getRepresentation();
			}
			catch (NKFException e)
			{	String msg = aHelper.formatMessage("MSG_NO_TRANSREPT", resp.getRepresentation().getClass().getName(),resp.getRepresentation().toString());
				rep=new ByteArrayRepresentation(msg.getBytes());
				mime="text/plain";
			}
		}
		else
		{	rep=resp.getRepresentation();
		}
		
		if (mime==null)
		{	mime="text/plain";
		}
		return new Object[]{rep,mime};
	}
		
	private RequestTrace getTraceForHash(EndpointMonitor aMonitor, String aHash, INKFRequestContext aHelper) throws NKFException
	{	RequestTrace trace = innerGetTraceForHash(aMonitor, aHash);
		if (trace==null)
		{	throw getTraceNotFoundException(aHelper);
		}
		return trace;
	}
	
	public static RequestTrace innerGetTraceForHash(EndpointMonitor aMonitor, String aHash)
	{
		long hashLong=Long.parseLong(aHash,16);
		int hashInt=(int)hashLong; //jdk can't parse negative hex integers- what an oversight but at least this is a workaround
		
		RequestTrace[] traces=aMonitor.getCapturedTraces();
		RequestTrace trace=null;
		for (int i=0; i<traces.length; i++)
		{	
			RequestTrace t=traces[i];
			RequestCapture r=t.getRoot();
			long time=t.getStart();
			IRequest req=r.getRequest();
			if (req==null)
			{	req=r.getRequestIn();
			}
			int hash2=(int)time ^ req.hashCode();
			if (hash2==hashInt)
			{	trace=t;
				break;
			}
		}
		return trace;
	}
	
	private LoadedTrace getLoadedTrace(String aHash)
	{	LoadedTrace result=null;
		if (mLoadedTraces!=null)
		{	for (int i=0; i<mLoadedTraces.size(); i++)
			{	LoadedTrace lt=(LoadedTrace)mLoadedTraces.get(i);
				if (lt.getHash().equals(aHash))
				{	result=lt;
					break;
				}
			}
		}
		return result;
	}
	
	public static RequestCapture getRequestCapture(RequestTrace aTrace,int aIndex) throws NKFException
	{	Object result= recurseCountRC(aTrace.getRoot(),0,aIndex);
		if (result instanceof RequestCapture)
		{	return (RequestCapture)result;
		}
		else
		{	throw new NKFException("RequestCapture not found in trace");
		}
	}
	
	private static Object recurseCountRC(RequestCapture aTrace, int aCount, int aIndex)
	{	if (aCount==aIndex)
		{	return aTrace;
		}
		else
		{	int d=aCount;
			for (Iterator i=aTrace.getSubRequests().iterator(); i.hasNext(); )
			{	
				RequestCapture subReq = (RequestCapture)i.next();
				if (subReq.getResponse().getMeta().getUserMetaData().getValue(VISUALIZER_IGNORE)==null)
				{	d++;
					Object result = recurseCountRC(subReq,d,aIndex);
					if (result instanceof RequestCapture)
					{	return result;
					}
					else
					{	d=((Integer)result).intValue();
					}
				}
			}
			return new Integer(d);
		}
	}

	/** Get indices for given captures
	 * @param aTrace
	 * @param aCaptures
	 * @return
	 */
	private int[] getCaptureIndices(RequestTrace aTrace, Object[] aCaptures)
	{	int[] result = new int[aCaptures.length];
		recurseCaptureIndices(aTrace.getRoot(), aCaptures, result,0);
		return result;
	}
	
	private int recurseCaptureIndices(RequestCapture aCapture, Object[] aCaptures, int[] aResults, int aIndex)
	{	try
		{	if (aCapture.getResponse().getMeta().getUserMetaData().getValue(VISUALIZER_IGNORE)!=null)
			{	return aIndex;
			}
		} catch (Exception e)
		{ ; }
		
		for (int i=0; i<aCaptures.length; i++)
		{	if (aCaptures[i]==aCapture)
			{	aResults[i]=aIndex;
			}
		}
		
		int index=aIndex+1;
		List children=aCapture.getSubRequests();
		for (int i=0; i<children.size(); i++)
		{	RequestCapture child = (RequestCapture)children.get(i);
			if (child!=null)
			{	index=recurseCaptureIndices(child,aCaptures,aResults,index);
			}
		}
		
		return index;
	}
	
	
	/**
	 * @param aWriter
	 * @param aIndex
	 * @param aRequest
	 * @param aTrace
	 * @param aRecurse 0=no, 1=single depth, 2=yes
	 * @param aFull
	 * @param aDMR
	 * @return
	 * @throws IOException
	 */
	private int writeRequest(Writer aWriter, int aIndex, RequestCapture aRequest, RequestTrace aTrace, int aRecurse, boolean aFull, IRepDeployedModules aDMR) throws IOException
	{
		IResponse response=aRequest.getResponse();
		
		if (response==null) return aIndex-1;
		
		if (response.getMeta().getUserMetaData().getValue(VISUALIZER_IGNORE)!=null)
		{	return aIndex-1;
		}
		
		IRequest req=aRequest.getRequestIn();
		if (req==null)
		{	return aIndex-1;
		}
		
		aWriter.write("<trace>");
		XMLUtils.writeEscaped(aWriter,"index",  Integer.toString(aIndex) );
		
		IRequest reqOut=aRequest.getRequest();
		
		boolean resolutionFailure=false;
		if (reqOut==null)
		{	resolutionFailure=true;
			Throwable t=(Throwable)response.getRepresentation();
			if (t==null)
			{	System.out.println("here");
			}
			ResolutionFailureException rfe=(ResolutionFailureException)t.getCause();
			reqOut=rfe.getRequest();			
		}
		
		XMLUtils.writeEscaped(aWriter,"identifier",  req.getIdentifier().toString() );
		int verb=reqOut.getVerb();
		String verbString=org.netkernel.request.impl.RequestFactory.verbString(verb);
		XMLUtils.writeEscaped(aWriter,"verb",  verbString ); 
		String repClassName= NULL_STRING;
		boolean isThrowable=false;
		if (aRequest!=null)
		{	if (response!=null)
			{	Object representation = response.getRepresentation();
				if (representation!=null)
				{	Class repClass=representation.getClass();
					repClassName=repClass.getName();
					String repHash=aDMR.hashForClass(repClass);
					if (repHash!=null)
					{	String repName=aDMR.metadataForHash(repHash).getName();
						XMLUtils.writeEscaped(aWriter,"responseHash", repHash );
						XMLUtils.writeEscaped(aWriter,"responseName", repName );
					}
				}
				isThrowable = representation instanceof Throwable;
			}
		}
		XMLUtils.writeEscaped(aWriter,"responseClass", repClassName );
		XMLUtils.writeEscaped(aWriter,"responseClassShort", Utils.shortClassName(repClassName) );
		XMLUtils.write(aWriter,"responseException", Boolean.toString(isThrowable) );
		XMLUtils.writeEscaped(aWriter,"async",  Boolean.toString(req.getRequestor()!=null && req.getRequestor().getClass()!=SynchronousRequestee.class) );
		
		

		IResponseMeta meta=response.getMeta();
		IInternalResponseMeta imeta=null;
		boolean expired=aRequest.isExpired();
		if (!(meta instanceof IInternalResponseMeta))
		{	// this happens in the situation of a response that needs subsequent transreption
			List sub=aRequest.getParent().getSubRequests();
			boolean foundThis=false;
			for (int i=0; i<sub.size(); i++)
			{	RequestCapture rc=(RequestCapture)sub.get(i);
				if (rc==aRequest)
				{	foundThis=true;
					continue;
				}
				if (foundThis && rc.getRequest()!=null
						&& rc.getRequest().getVerb()==IRequest.VERB_TRANSREPT
						&& aRequest.getRequest()!=null
						&& rc.getRequest().getIdentifier().equals(aRequest.getRequest().getIdentifier())
						&& rc.getRequest().getValue(IRequest.TYPE_SOURCED_RESPONSE)==aRequest.getResponse()
					)
				{	imeta=(IInternalResponseMeta)rc.getResponse().getMeta();
					expired=rc.isExpired();
					aWriter.write("<transreptMeta/>");
					break;
				}
			}
		}
		else
		{	imeta=(IInternalResponseMeta)meta;
		}
		
		
		
		XMLUtils.write(aWriter,"isExpired", Boolean.toString(expired) );
		
		if (aFull)
		{
			int priority=req.getPriority();
			String priorityString;
			if (priority==IRequest.PRIORITY_MIN)
			{	priorityString="LOWER";
			}
			else if (priority==IRequest.PRIORITY_MAX)
			{	priorityString="HIGHER";
			}
			else
			{	priorityString="NORMAL";
			}
			XMLUtils.writeEscaped(aWriter,"priority",  priorityString );
			
			Class repClass=req.getRepresentationClass();
			String repHash=aDMR.hashForClass(repClass);
			if (repHash!=null)
			{	String repName=aDMR.metadataForHash(repHash).getName();
				XMLUtils.writeEscaped(aWriter,"rrH", repHash );
				XMLUtils.writeEscaped(aWriter,"rrN", repName );
			}
			XMLUtils.writeEscaped(aWriter,"rrC",  repClass.getName() );
			
			
			
			
			aWriter.write("<headers>");
			IRequestResponseFields userMeta=req.getHeaders();
			int userMetaSize=userMeta.size();
			for (int j=0; j<userMetaSize; j++ )
			{	String key = userMeta.getKey(j);
				aWriter.write("<value key='"+key+"'>");
				aWriter.write(XMLUtils.escape(userMeta.getValue(j).toString()));
				aWriter.write("</value>");
			}
			aWriter.write("</headers>");			
			
			IRequestScopeLevel ctx = req.getRequestScope();
			aWriter.write("<reqctx>");
			writeScope(aWriter,ctx,aDMR,true);
			aWriter.write("</reqctx>");
			
			if (verb==IRequest.VERB_SINK || verb==IRequest.VERB_TRANSREPT || verb==IRequest.VERB_NEW)
			{	IResponse sourced=(IResponse)reqOut.getValue(IRequest.TYPE_SOURCED_RESPONSE);
				if (sourced!=null)
				{	aWriter.write("<primary>");				
					repClass=sourced.getRepresentation().getClass();
					String primaryClassname;
					if (repClass==null)
					{	primaryClassname="null";
					}
					else
					{	primaryClassname = sourced.getRepresentation().getClass().getName();
						repHash=aDMR.hashForClass(repClass);
						if (repHash!=null)
						{	String repName=aDMR.metadataForHash(repHash).getName();
							XMLUtils.writeEscaped(aWriter,"pH", repHash );
							XMLUtils.writeEscaped(aWriter,"pN", repName );
						}
					}
					XMLUtils.writeEscaped(aWriter,"pC",  primaryClassname );				
				
					aWriter.write("</primary>");
				}
			}			
		}
	
		String epHomeHash=null;
		if (resolutionFailure)
		{	// resolution failure
			aWriter.write("<rfe/>");
		}
		else
		{	IEndpoint endpoint = aRequest.getEndpoint();
			if (endpoint==null)
			{	aWriter.write("<cached/>");
			}
			else if (endpoint==EndpointMonitor.sInternalDependency)
			{	aWriter.write("<internalDependency/>");
			}
			else
			{	XMLUtils.writeEscaped(aWriter,"endpointShort",  endpointShort(endpoint) );
				
				IRequestScopeLevel ctx=reqOut.getRequestScope();
				aWriter.write("<resctx>");
				writeScope(aWriter,ctx,aDMR,aFull);
				aWriter.write("</resctx>");
				
				//attempt to locate endpoints home context
				ISpace epCtx;
				if (endpoint instanceof ISpace)
				{	epCtx=(ISpace)endpoint;
					epHomeHash=aDMR.hashForComponent(endpoint);
				}
				else
				{	String physicalEndpointHash=aDMR.hashForComponent(endpoint);
					if (physicalEndpointHash!=null)
					{	int i=physicalEndpointHash.lastIndexOf('/');
						String contextHash=physicalEndpointHash.substring(0,i);
						epHomeHash=contextHash;
						epCtx=(ISpace)aDMR.componentForHash(contextHash);
					}
					else
					{	epCtx = reqOut.getRequestScope().getSpace();
						epHomeHash=aDMR.hashForComponent(epCtx);
					}
				}
				XMLUtils.writeEscaped(aWriter,"endpointHomeContext",  epCtx.toString() );
				if (epHomeHash!=null) XMLUtils.writeEscaped(aWriter,"endpointHomeHash",  epHomeHash );
				
			}
			
			long total = aTrace.getEnd()-aTrace.getStart();
			if (total==0) total=1;
			
			if (mTimings)
			{	float divisor = (float)getKernel().getTimer().getRelativeTimeDivisor();
				float t=aRequest.getDuration()/divisor;
				float percent = 100.0f * t /(float)(total);
				if (percent>100) percent=100L;
				
				XMLUtils.writeEscaped(aWriter,"t-percent",  sDecimalFormat.format(percent) );
				XMLUtils.writeEscaped(aWriter,"t-duration",  sDecimalFormat.format(t) );
			}
			
			if (mTimings)
			{	long t=aRequest.getElapsed();
				long percent = 100L * t /(total);
				if (percent>100) percent=100L;
				XMLUtils.writeEscaped(aWriter,"e-percent",  Long.toString(percent) );
				XMLUtils.writeEscaped(aWriter,"e-duration",  Long.toString(t) );
			}
			
			if (aFull)
			{
				if (endpoint!=null && endpoint!=EndpointMonitor.sInternalDependency)
				{	XMLUtils.writeEscaped(aWriter,"endpoint",  endpoint.getClass().getName() );
					XMLUtils.writeEscaped(aWriter,"endpointToString",  endpoint.toString() );
					String physicalEndpointHash=aDMR.hashForComponent(endpoint);
					if (physicalEndpointHash!=null)
					{	XMLUtils.writeEscaped(aWriter,"endpointHash",  physicalEndpointHash );
					}
				}
				IIdentifier logicalId=(IIdentifier)aRequest.getRequest().getValue(IRequest.TYPE_RESOLVED_ENDPOINT_IDENTIFIER);
				if (logicalId!=null)
				{	XMLUtils.write(aWriter,"logicalEndpoint", logicalId.toString() );
					if (epHomeHash!=null)
					{	String logicalHash=epHomeHash+"/"+logicalId;
						XMLUtils.write(aWriter,"logicalHash", logicalHash );
					}
				}
						
							
				aWriter.write("<usermeta>");
				IRequestResponseFields userMeta=meta.getUserMetaData();
				int userMetaSize=userMeta.size();
				for (int j=0; j<userMetaSize; j++ )
				{	String key = userMeta.getKey(j);
					if (!key.equals(IResponseMeta.KEY_ADDITIONAL_DEPENDENCIES))
					{	aWriter.write("<value key='"+key+"'>");
						Object value=userMeta.getValue(j);
						String valueString;
						if (value==null)
						{	valueString=NULL_STRING;
						}
						else if (value.getClass().isArray())
						{	valueString=Utils.arrayToString((Object[])value, 64);
						}
						else
						{	valueString=value.toString();
						}
						aWriter.write(XMLUtils.escape(valueString));
						aWriter.write("</value>");
					}
				}
				aWriter.write("</usermeta>");
			}

			
			
			int depth=0;
			if (imeta!=null)
			{
				XMLUtils.write(aWriter,"expiry", ExpiryFactory.toString(imeta.getExpiry()) );
				XMLUtils.writeEscaped(aWriter,"cost", Integer.toString(imeta.getTotalCost()) );
				XMLUtils.writeEscaped(aWriter,"costl", Integer.toString(imeta.getLocalCost()) );
				/*
				XMLUtils.writeEscaped(aWriter,"costm", Integer.toString(im.getCostDeterminantsLength()) );
				IInternalResponseMeta[] dim=im.getCostDeterminants();
				int dimc=0;
				for (int i=0; i<im.getCostDeterminantsLength(); i++)
				{	dimc+=dim[i].getLocalCost();
				}
				XMLUtils.writeEscaped(aWriter,"costd", Integer.toString(dimc) );
				*/
				
				depth=req.getRequestScope().getDepth()-imeta.getUnresolvedScope();
				depth=depth>0?depth:0;
				XMLUtils.writeEscaped(aWriter,"scope", Integer.toString(depth) );
				
				XMLUtils.writeEscaped(aWriter,"reqctxDepth", Integer.toString(req.getRequestScope().getDepth()) );
				
			}
			
		
			//isCachable test
			boolean implicitTransrept=false;
			if (verb==IRequest.VERB_TRANSREPT)
			{	RequestCapture last=null;
				List sub=aRequest.getParent().getSubRequests();
				for (int i=0; i<sub.size(); i++)
				{	RequestCapture rc=(RequestCapture)sub.get(i);
					if (rc==aRequest)
					{	if (last!=null &&last.getRequestIn()==rc.getRequestIn())
						{	implicitTransrept=true;
						}
						break;
					}
					last=rc;
				}
			}
			
			String cachabilityStatus="Response is cachable";
			String cachabilityTool=null;
			
			boolean isCachable = true;
			if (expired)
			{	isCachable=false;
				cachabilityStatus="Response is expired ";
				cachabilityTool="visExpiryDeterminants";
			}
			if (isCachable && (verb!=IRequest.VERB_SOURCE&&verb==IRequest.VERB_EXISTS&&verb==IRequest.VERB_META&&implicitTransrept))
			{	isCachable=false;
				cachabilityStatus=verbString+" requests are not cachable";
			}
			
			if (isCachable)
			{
				IRequestResponseFields rh=aRequest.getRequestIn().getHeaders();
				if (rh.getValue(IRequest.HEADER_FORGET_DEPENDENCIES)!=null || rh.getValue(IRequest.HEADER_NO_CACHE)!=null)
				{	isCachable=false;
					cachabilityStatus="Request header inhibits caching";
				}
				else
				{	rh=meta.getUserMetaData();
					if (rh.getValue(IResponseMeta.KEY_NO_CACHE)!=null)
					{	isCachable=false;
						cachabilityStatus="Response header inhibits caching";
					}
				}
			}
			boolean maybeCachable=isCachable && depth>0;
			if (maybeCachable)
			{	cachabilityStatus="Response depends upon request scope so retrieval may not be possible";
				cachabilityTool="visScopeDeterminants";
			}
			
			if (aRequest.getEndpoint()==null)
			{	maybeCachable=false;
				isCachable=true;
				cachabilityStatus="Response is retrieved from cache";
			}
			
			String cs=maybeCachable?"maybe":(isCachable?"true":"false");
			XMLUtils.write(aWriter,"isCachable",cs );
			XMLUtils.write(aWriter,"cachableStatus",cachabilityStatus );
			if (mTimings)
			{
				if (cachabilityTool!=null)
				{	XMLUtils.write(aWriter,"cachableTool",cachabilityTool );
				}
			}
			
			
			
			int recurse=aRecurse==2?2:0;
			for (Iterator j=aRequest.getSubRequests().iterator(); j.hasNext(); )
			{	RequestCapture subReq = (RequestCapture)j.next();
			
				if (aRecurse==0 || (subReq.getEndpoint()==EndpointMonitor.sInternalDependency && !aFull))
				{	aIndex=recurseCountRequests(aIndex+1,subReq);
				}
				else
				{	aIndex=writeRequest(aWriter,aIndex+1,subReq,aTrace,recurse,aFull,aDMR);
				}
			}
		}

		aWriter.write("</trace>");
		return aIndex;
	}
	
	private int recurseCountRequests(int aIndex, RequestCapture aRequest)
	{	for (Iterator j=aRequest.getSubRequests().iterator(); j.hasNext(); )
		{	RequestCapture subReq = (RequestCapture)j.next();
			aIndex=recurseCountRequests(aIndex+1,subReq);
		}
		return aIndex;
	}
	
	private static void writeScope(Writer aWriter, IRequestScopeLevel ctx, IRepDeployedModules aDMR, boolean aAll) throws IOException
	{	while(ctx!=null)
		{	ISpace space=ctx.getSpace();
			String spaceHash=aDMR.hashForComponent(space);
			if (spaceHash==null)
			{	if (space instanceof StandardSpace)
				{	ISpace protoSpace=((StandardSpace)space).getProtoSpace();
					if (protoSpace!=null)
					{	space=protoSpace;
						spaceHash=aDMR.hashForComponent(space);
					}
				}
			}
			aWriter.write("<level>");
			XMLUtils.writeEscaped(aWriter,"space", space.toString()  );
			if (spaceHash!=null) XMLUtils.writeEscaped(aWriter,"hash", spaceHash  );
			ctx=ctx.getParent();
			aWriter.write("</level>");
			if (!aAll) break;
		}
	}
	
	private String endpointShort(Object aEndpoint)
	{	final Class[] noParameters = new Class[0];
		String result=null;
		try
		{	if (aEndpoint.getClass().getMethod("toString", noParameters).getDeclaringClass()!=Object.class)
			{	result = aEndpoint.toString();
			}
		} catch (NoSuchMethodException e) {;}
		if (result==null)
		{	result = Utils.shortClassName(aEndpoint.getClass());
		}
		return result;
	}
	
	
	private void generateSaveResponse(INKFRequestContext aHelper, EndpointMonitor aMonitor, String aTraceId, IRepDeployedModules aDMR) throws Exception
	{
		RequestTrace trace = getTraceForHash(aMonitor,aTraceId,aHelper);
		if (trace.getRoot().getResponse()==null)
		{	throw getTraceNotFoundException(aHelper);
		}
		IReadableBinaryStreamRepresentation traceString = generateTraceView(trace, aTraceId,aDMR,false);
		
		ByteArrayOutputStream os = new ByteArrayOutputStream(4096);
		NoCloseZipOutputStream zos = new NoCloseZipOutputStream(os);
		try
		{
			zos.putNextEntry(new ZipEntry("trace.xml"));
			Writer w = new OutputStreamWriter(zos);
			writeTraceSummary(w,trace);
			w.flush();
			zos.closeEntry();
			
			zos.putNextEntry(new ZipEntry("index.xml"));
			traceString.write(zos);
			zos.closeEntry();
			
			int i=0;
			while(true)
			{	String is = Integer.toString(i);
				try
				{	IReadableBinaryStreamRepresentation detailString = generateDetailViewString(aHelper, trace, aTraceId, is, aDMR);
					zos.putNextEntry(new ZipEntry("detail/"+is+".xml"));
					detailString.write(zos);
					zos.closeEntry();
					
					RequestCapture rc = getRequestCapture(trace,i);
					Object[] ram = getResponseAndMime(aHelper, rc,false);
					
					zos.putNextEntry(new ZipEntry("response/"+is));
					if (ram[0]!=null)
					{	((IBinaryStreamRepresentation)ram[0]).write(zos);
					}
					else
					{	zos.write(NULL_STRING.getBytes());
					}
					zos.closeEntry();
					
					zos.putNextEntry(new ZipEntry("response/"+is+".mime"));
					String mime=(String)ram[1];
					zos.write(mime.getBytes("UTF-8"));
					zos.closeEntry();
					
					ram = getResponseAndMime(aHelper, rc,true);
					if (ram!=null)
					{
						zos.putNextEntry(new ZipEntry("primary/"+is));
						if (ram[0]!=null)
						{	((IBinaryStreamRepresentation)ram[0]).write(zos);
						}
						else
						{	zos.write(NULL_STRING.getBytes());
						}
						zos.closeEntry();
						
						zos.putNextEntry(new ZipEntry("primary/"+is+".mime"));
						mime=(String)ram[1];
						zos.write(mime.getBytes("UTF-8"));
						zos.closeEntry();
					}
					
					i++;
				}
				catch (NKFException e)
				{	// no more
					break;
				}
				
			}
		}
		finally
		{	zos.reallyClose();
		}
		
		ByteArrayRepresentation rep = new ByteArrayRepresentation(os);
		INKFResponse response=aHelper.createResponseFrom(rep);
		response.setMimeType("application/zip");		
	}
	
	private void deleteTrace(EndpointMonitor aMonitor, String aTraceId)
	{
		if (mLoadedTraces!=null)
		{	for (Iterator<LoadedTrace> i = mLoadedTraces.iterator(); i.hasNext(); )
			{	LoadedTrace lt=i.next();
				if (lt.getHash().equals(aTraceId))
				{	i.remove();
					return;
				}
			}
		}
		
		long hashLong=Long.parseLong(aTraceId,16);
		int hashInt=(int)hashLong; //jdk can't parse negative hex integers- what an oversight but at least this is a workaround
		
		RequestTrace[] traces=aMonitor.getCapturedTraces();
		for (int i=0; i<traces.length; i++)
		{	
			RequestTrace t=traces[i];
			RequestCapture r=t.getRoot();
			long time=t.getStart();
			IRequest req=r.getRequest();
			if (req==null)
			{	req=r.getRequestIn();
			}
			int hash2=(int)time ^ req.hashCode();
			if (hash2==hashInt)
			{	IRequest rootReq=t.getRootRequest();
				aMonitor.removeCapturedTrace(rootReq);
				return;
			}
		}		
		
	}
	
	private static class NoCloseZipOutputStream extends ZipOutputStream
	{
		public NoCloseZipOutputStream(OutputStream aStream)
		{	super(aStream);
		}
		
		public void close()
		{
		}
		
		public void reallyClose() throws IOException
		{	super.close();
		}
	}
	
}