package org.ten60.netkernel.visualizer;

import org.w3c.dom.*;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.util.Utils;
import org.netkernel.layer0.util.XMLReadable;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.netkernel.xml.xda.IXDAReadOnly;
import org.netkernel.xml.xda.IXDAReadOnlyIterator;
import org.netkernel.xml.xda.XPathLocationException;

public class VisualizerExplorerViewAccessor extends StandardAccessorImpl
{
	public VisualizerExplorerViewAccessor()
	{	declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext aHelper) throws Exception
	{
		INKFRequestReadOnly req=aHelper.getThisRequest();
		String action2=req.getArgumentValue("action");
		if (action2.equals("root"))
		{
			IHDSNode param = (IHDSNode)aHelper.source("httpRequest:/params",IHDSNode.class);
			Object action=param.getFirstValue("action2");
			if ("clear".equals(action))
			{	aHelper.source("active:visualizerClear");
			}
			else if ("start".equals(action))
			{	aHelper.source("active:visualizerStart");
			}
			else if ("stop".equals(action))
			{	aHelper.source("active:visualizerStop");
			}
			else if ("upload".equals(action))
			{	IBinaryStreamRepresentation rep=(IBinaryStreamRepresentation)aHelper.source("httpRequest:/upload/uploaded",IBinaryStreamRepresentation.class);
				
				INKFRequest subreq=aHelper.createRequest("active:visualizerLoad");
				subreq.addArgumentByValue("data", rep);
				aHelper.issueRequest(subreq);
			}
			else if ("cache".equals(action))
			{	aHelper.getKernelContext().getKernel().getRepresentationCache().clear();
			}
			
			String matchString=(String)param.getFirstValue("match");
			if (matchString!=null)
			{	INKFRequest subreq=aHelper.createRequest("active:visualizerSetFilter");
				subreq.addArgumentByValue("data", matchString);
				aHelper.issueRequest(subreq);
			}
			generateTracesView(aHelper);
		}
		else if (action2.equals("deleteTrace"))
		{	String traceId=req.getArgumentValue("entity");
			deleteTrace(aHelper,traceId);
			generateTracesView(aHelper);
		}
		else if (action2.equals("viewTrace"))
		{	String entity=req.getArgumentValue("entity");
			String[] entityBits=Utils.splitString(entity, "/");
			String root=entityBits.length>1?entityBits[1]:null;
			generateTraceView(aHelper,entityBits[0],root);
		}
		else if (action2.equals("saveTrace"))
		{	String entity=req.getArgumentValue("entity");
			String[] entityBits=Utils.splitString(entity, "/");
			generateSaveResponse(aHelper,entityBits[0]);
		}
		else if (action2.equals("viewRequest"))
		{	String entity=req.getArgumentValue("entity");
			String[] entityBits=Utils.splitString(entity, "/");
			generateDetailView(aHelper,entityBits[0],entityBits[1]);
		}
		else if (action2.equals("viewResponse"))
		{	String entity=req.getArgumentValue("entity");
			String[] entityBits=Utils.splitString(entity, "/");
			generateResponseView(aHelper,entityBits[0],entityBits[1]);
		}
		else if (action2.equals("viewPrimary"))
		{	String entity=req.getArgumentValue("entity");
			String[] entityBits=Utils.splitString(entity, "/");
			generatePrimaryView(aHelper,entityBits[0],entityBits[1]);
		}
		else if (action2.equals("viewCallstack"))
		{	String entity=req.getArgumentValue("entity");
			String[] entityBits=Utils.splitString(entity, "/");
			generateCallstackView(aHelper,entityBits[0],entityBits[1]);
		}
	}
	
	private void generateTracesView(INKFRequestContext aHelper) throws Exception
	{
		INKFRequest req = aHelper.createRequest("active:xslt");
		req.addArgument("operator","res:/estylesheets/styleTraces.xsl");
		req.addArgument("operand","active:visualizerData");
		req.setRepresentationClass(aHelper.getThisRequest().getRepresentationClass());
		
		INKFResponseReadOnly subResp=aHelper.issueRequestForResponse(req);

		req=aHelper.createRequest("active:xrl2");
		req.addArgumentFromResponse("template",subResp);
		subResp=aHelper.issueRequestForResponse(req);	
		
		INKFResponse respOut=aHelper.createResponseFrom(subResp);
		respOut.setHeader("WrappedControlPanel", true);
	}
	
	private void generateTraceView(INKFRequestContext aHelper, String aTraceId, String aRequestId) throws Exception
	{
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.setRepresentationClass(Document.class);
		Document data = (Document)aHelper.issueRequest(subreq);

		INKFRequest req = aHelper.createRequest("active:xsltc");
		req.addArgument("operator","res:/estylesheets/styleTrace.xsl");
		if (aRequestId!=null)
		{	req.addArgumentByValue("request",aRequestId);
		}
		req.addArgumentByValue("operand",data);
		req.setRepresentationClass(aHelper.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly subResp=aHelper.issueRequestForResponse(req);

		INKFResponse resp=aHelper.createResponseFrom(subResp);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);		
		resp.setHeader("WrappedControlPanel", true);
	}
	
	private void generateCallstackView(INKFRequestContext aHelper, String aTraceId, String aRequestId) throws Exception
	{
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.setRepresentationClass(Document.class);
		Document data = (Document)aHelper.issueRequest(subreq);

		INKFRequest req = aHelper.createRequest("active:xsltc");
		req.addArgument("operator","res:/estylesheets/styleTrace.xsl");
		req.addArgumentByValue("request",aRequestId);
		req.addArgumentByValue("operand",data);
		req.addArgumentByValue("mode","callstack");
		req.setRepresentationClass(aHelper.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly subResp=aHelper.issueRequestForResponse(req);

		INKFResponse resp=aHelper.createResponseFrom(subResp);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);		
		resp.setHeader("WrappedControlPanel", true);
	}

	private void generateDetailView(INKFRequestContext aHelper, String aTraceId, String aRequestId) throws Exception
	{	

		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.addArgument("detail", aRequestId);
		subreq.setRepresentationClass(Document.class);
		Document data = (Document)aHelper.issueRequest(subreq);

		//style subrequests
		INKFRequest req = aHelper.createRequest("active:xsltc");
		req.addArgument("operator","res:/estylesheets/styleTrace.xsl");
		req.addArgumentByValue("request",aRequestId);
		req.addArgumentByValue("mode","subrequests");
		req.addArgumentByValue("operand",data);
		req.setRepresentationClass(aHelper.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly subResp=aHelper.issueRequestForResponse(req);

		//style detail
		INKFRequest req2 = aHelper.createRequest("active:xslt");
		req2.addArgument("operator","res:/estylesheets/styleDetail.xsl");
		req2.addArgumentByValue("operand",data);
		req2.addArgumentFromResponse("subrequests", subResp);
		req2.setRepresentationClass(aHelper.getThisRequest().getRepresentationClass());
		subResp=aHelper.issueRequestForResponse(req2);

		INKFResponse resp=aHelper.createResponseFrom(subResp);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);			
		resp.setHeader("WrappedControlPanel", true);
	}
	
	private void generateResponseView(INKFRequestContext aHelper, String aTraceId, String aRequestId) throws Exception
	{ 
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.addArgument("response", aRequestId);
		INKFResponseReadOnly subResp = aHelper.issueRequestForResponse(subreq);
		aHelper.createResponseFrom(subResp);
	}
	
	private void generatePrimaryView(INKFRequestContext aHelper, String aTraceId, String aRequestId) throws Exception
	{ 
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.addArgument("primary", aRequestId);
		INKFResponseReadOnly subResp = aHelper.issueRequestForResponse(subreq);
		aHelper.createResponseFrom(subResp);
	}
	
	private void generateSaveResponse(INKFRequestContext aHelper, String aTraceId) throws Exception
	{
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.addArgument("save", "/save");
		INKFResponseReadOnly subResp = aHelper.issueRequestForResponse(subreq);
		
		aHelper.createResponseFrom(subResp);
		aHelper.sink("httpResponse:/header/Content-Disposition", "attachment; filename=netkernel_trace_"+aTraceId+".zip");
	}
	
	private void deleteTrace(INKFRequestContext aHelper, String aTraceId) throws Exception
	{
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action", "Delete");
		subreq.addArgument("data", aTraceId);
		aHelper.issueRequestForResponse(subreq);
		
		//redirect to visualizer root
		aHelper.sink("httpResponse:/redirect","/tools/ae/view/visualizer");
	}
}
